Class: CarinForBlueButtonTestKit::Generator::GroupMetadataExtractor

Inherits:
Object
  • Object
show all
Defined in:
lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(resource_capabilities, profile_url, ig_metadata, ig_resources) ⇒ GroupMetadataExtractor

Returns a new instance of GroupMetadataExtractor.



13
14
15
16
17
18
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 13

def initialize(resource_capabilities, profile_url, , ig_resources)
  self.resource_capabilities = resource_capabilities
  self.profile_url = profile_url
  self. = 
  self.ig_resources = ig_resources
end

Instance Attribute Details

#ig_metadata ⇒ Object

Returns the value of attribute ig_metadata.



11
12
13
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 11

def 
  @ig_metadata
end

#ig_resources ⇒ Object

Returns the value of attribute ig_resources.



11
12
13
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 11

def ig_resources
  @ig_resources
end

#profile_url ⇒ Object

Returns the value of attribute profile_url.



11
12
13
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 11

def profile_url
  @profile_url
end

#resource_capabilities ⇒ Object

Returns the value of attribute resource_capabilities.



11
12
13
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 11

def resource_capabilities
  @resource_capabilities
end

Instance Method Details

#base_name ⇒ Object



103
104
105
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 103

def base_name
  profile_url.split('StructureDefinition/').last
end

#bindings ⇒ Object



208
209
210
211
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 208

def bindings
  @bindings ||=
    .terminology_bindings
end

#class_name ⇒ Object



111
112
113
114
115
116
117
118
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 111

def class_name
  base_name
    .split('-')
    .map(&:capitalize)
    .join
    .gsub('CARIN4BB', "CARIN4BB#{.reformatted_version}")
    .concat('Sequence')
end

#conformance_expectation ⇒ Object



132
133
134
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 132

def conformance_expectation
  resource_capabilities.extension.first.valueCode
end

#group_metadata ⇒ Object



20
21
22
23
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 20

def 
  @group_metadata ||=
    GroupMetadata.new()
end

#group_metadata_hash ⇒ Object



25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 25

def 
  @group_metadata_hash ||=
    {
      name:,
      class_name:,
      version:,
      reformatted_version:,
      resource:,
      conformance_expectation:,
      profile_url:,
      profile_name:,
      profile_version:,
      title:,
      short_description:,
      interactions:,
      operations:,
      searches:,
      search_definitions:,
      include_params:,
      revincludes:,
      required_concepts:,
      must_supports:,
      mandatory_elements:,
      bindings:,
      references:
    }

  mark_mandatory_and_must_support_searches

  @group_metadata_hash
end

#include_params ⇒ Object



186
187
188
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 186

def include_params
  resource_capabilities.searchInclude || []
end

#interactions ⇒ Object



153
154
155
156
157
158
159
160
161
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 153

def interactions
  @interactions ||=
    resource_capabilities.interaction.map do |interaction|
      {
        code: interaction.code,
        expectation: interaction.extension.first.valueCode # TODO: fix expectation extension finding
      }
    end
end

#mandatory_elements ⇒ Object



223
224
225
226
227
228
229
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 223

def mandatory_elements
  @mandatory_elements ||=
    profile_elements
    .select { |element| element.min.positive? }
    .map(&:path)
    .uniq
end

#mark_mandatory_and_must_support_searches ⇒ Object



57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 57

def mark_mandatory_and_must_support_searches
  searches.each do |search|
    search[:names_not_must_support_or_mandatory] = search[:names].reject do |name|
      full_paths = search_definitions[name.to_sym][:full_paths]
      any_must_support_elements = must_supports[:elements].any? do |element|
        full_must_support_paths = ["#{resource}.#{element[:original_path]}", "#{resource}.#{element[:path]}"]

        full_paths.any? do |path|
          # allow for non-choice, choice types, and _id
          name == '_id' || full_must_support_paths.include?(path) || full_must_support_paths.include?("#{path}[x]")
        end
      end

      any_must_support_slices = must_supports[:slices].any? do |slice|
        # only handle type slices because that is all we need for now
        # for a slice like Observation.effective[x]:effectiveDateTime, the search parameter's expression could be
        # either Observation.effective or Observation.effectiveDateTime.
        if slice[:discriminator] && slice[:discriminator][:type] == 'type'
          full_must_support_path = "#{resource}.#{slice[:path].sub('[x]', slice[:discriminator][:code])}"
          base_must_support_path = "#{resource}.#{slice[:path].sub('[x]', '')}"

          full_paths.intersection([full_must_support_path, base_must_support_path]).present?
        else
          false
        end
      end

      any_mandatory_elements = mandatory_elements.any? do |element|
        full_paths.include?(element)
      end

      any_must_support_elements || any_must_support_slices || any_mandatory_elements
    end

    search[:must_support_or_mandatory] = search[:names_not_must_support_or_mandatory].empty?
  end
end

#must_support_metadata_extractor ⇒ Object



213
214
215
216
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 213

def 
  @must_support_metadata_extractor ||=
    Inferno::DSL::MustSupportMetadataExtractor.new(profile_elements, profile, resource, ig_resources)
end

#must_supports ⇒ Object



218
219
220
221
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 218

def must_supports
  @must_supports ||=
    .must_supports
end

#name ⇒ Object



107
108
109
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 107

def name
  base_name.tr('-', '_')
end

#operations ⇒ Object



163
164
165
166
167
168
169
170
171
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 163

def operations
  @operations ||=
    resource_capabilities.operation.map do |operation|
      {
        code: operation.name,
        expectation: operation.extension.first.valueCode # TODO: fix expectation extension finding
      }
    end
end

#profile ⇒ Object



95
96
97
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 95

def profile
  @profile ||= ig_resources.profile_by_url(profile_url)
end

#profile_elements ⇒ Object



99
100
101
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 99

def profile_elements
  @profile_elements ||= profile.snapshot.element
end

#profile_name ⇒ Object



136
137
138
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 136

def profile_name
  profile.title.gsub('  ', ' ')
end

#profile_version ⇒ Object



140
141
142
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 140

def profile_version
  profile.version
end

#references ⇒ Object



231
232
233
234
235
236
237
238
239
240
241
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 231

def references
  @references ||=
    profile_elements
    .select { |element| element.type&.first&.code == 'Reference' }
    .map do |reference_definition|
      {
        path: reference_definition.path,
        profiles: reference_definition.type.first.targetProfile
      }
    end
end

#reformatted_version ⇒ Object



124
125
126
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 124

def reformatted_version
  .reformatted_version
end

#required_concepts ⇒ Object



194
195
196
197
198
199
200
201
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 194

def required_concepts
  return [] if resource == 'Observation'

  profile_elements
    .select { |element| element.type&.any? { |type| type.code == 'CodeableConcept' } }
    .select { |element| element.binding&.strength == 'required' }
    .map { |element| element.path.gsub("#{resource}.", '').gsub('[x]', 'CodeableConcept') }
end

#resource ⇒ Object



128
129
130
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 128

def resource
  resource_capabilities.type
end

#revincludes ⇒ Object



190
191
192
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 190

def revincludes
  resource_capabilities.searchRevInclude || []
end

#search_definitions ⇒ Object



182
183
184
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 182

def search_definitions
  @search_definitions ||= .search_definitions
end

#search_metadata_extractor ⇒ Object



173
174
175
176
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 173

def 
  @search_metadata_extractor ||=
    SearchMetadataExtractor.new(resource_capabilities, ig_resources, resource, profile_elements)
end

#searches ⇒ Object



178
179
180
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 178

def searches
  @searches ||= .searches
end

#short_description ⇒ Object



149
150
151
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 149

def short_description
  "Verify support for the server capabilities required by the #{profile_name}."
end

#terminology_binding_metadata_extractor ⇒ Object



203
204
205
206
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 203

def 
  @terminology_binding_metadata_extractor ||=
    TerminologyBindingMetadataExtractor.new(profile_elements, ig_resources, resource)
end

#title ⇒ Object



144
145
146
147
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 144

def title
  puts profile.title
  profile.title.gsub(/C4BB\s*/, '').gsub(/\s*Profile/, '').strip
end

#version ⇒ Object



120
121
122
# File 'lib/carin_for_blue_button_test_kit/generator/group_metadata_extractor.rb', line 120

def version
  .ig_version
end